from django.conf.urls import url, include

from .api import api_v1_router
from .views import OperationsViewSet

urlpatterns = (
    url(
        r"^pzone/(?P<pzone_pk>\d+)/operations/$",
        OperationsViewSet.as_view(),
        name="pzone_operations"
    ),
    url(
        r"^pzone/(?P<pzone_pk>\d+)/operations/(?P<operation_pk>\d+)/$",
        OperationsViewSet.as_view(),
        name="pzone_operations_delete"
    ),
    url(r"^", include(api_v1_router.urls)),       # noqa
)



from django.core.management.base import BaseCommand
from django.utils import timezone

from bulbs.content.models import Content, FeatureType
from bulbs.content.tasks import post_to_instant_articles_api


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('feature', nargs="+", type=str)

    def handle(self, *args, **options):
        feature_types = FeatureType.objects.filter(instant_article=True)

        feature = options['feature'][0]
        if feature:
            feature_types = feature_types.filter(slug=feature)

        for ft in feature_types:
                # All unmigrated published content belonging to feature type
                content = Content.objects.filter(
                    feature_type=ft,
                    published__isnull=False,
                    published__lte=timezone.now(),
                    instant_article_id__isnull=True)

                for c in content:
                    post_to_instant_articles_api.delay(c.id)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import djbetty.fields


class Migration(migrations.Migration):

    dependencies = [
        ('poll', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='answer',
            name='answer_image',
            field=djbetty.fields.ImageField(default=None, null=True, blank=True),
        ),
        migrations.AddField(
            model_name='poll',
            name='answer_type',
            field=models.TextField(default=b'text', blank=True),
        ),
        migrations.AddField(
            model_name='poll',
            name='poll_image',
            field=djbetty.fields.ImageField(default=None, null=True, blank=True),
        ),
    ]



from django.conf.urls import url, patterns
from django.views.decorators.cache import cache_control

from .viewsets import ReadOnlyNotificationViewSet


urlpatterns = patterns(
    "",
    url(
        r"^notifications.json",
        cache_control(max_age=300)(
            ReadOnlyNotificationViewSet.as_view({'get': 'list'})
        ), name="notification-all"),
)



from django import template
from djbetty.templatetags.betty import AnonymousImageField


register = template.Library()


def image_url(image_id):
    image = AnonymousImageField(image_id)
    return image.get_crop_url(width=300, ratio="3x1")


def utm_url(absolute_uri, content):
    return "//" + absolute_uri + content.get_absolute_url() + "?utm_campaign=InstantArticles&utm_source=Facebook&utm_medium=SponsoredPromo"



from django.conf import settings
from django.core.exceptions import ValidationError
from django.db.models.signals import pre_delete
from django.db import models
from django.template.defaultfilters import slugify

from elasticsearch import Elasticsearch
from json_field import JSONField

from bulbs.content.custom_search import custom_search_model
from bulbs.content.models import Content
from bulbs.content.mixins import DetailImageMixin
from bulbs.utils.methods import (datetime_to_epoch_seconds,
                                 today_as_utc_datetime,
                                 is_valid_digit)
from .managers import SpecialCoverageManager


es = Elasticsearch(settings.ES_CONNECTIONS["default"]["hosts"])


class SpecialCoverage(DetailImageMixin, models.Model):
    name = models.CharField(max_length=255, unique=True)
    slug = models.SlugField(max_length=255, blank=True, editable=True, unique=True)
    description = models.TextField(default="", blank=True)
    query = JSONField(default={}, blank=True)
    videos = JSONField(default=[], blank=True)
    super_features = JSONField(default=[], blank=True)
    active = models.BooleanField(default=False)
    promoted = models.BooleanField(default=False)
    start_date = models.DateTimeField(blank=True, null=True)
    end_date = models.DateTimeField(blank=True, null=True)
    # Tunic Campaign ID
    # NOTE: Don't want to accidentally overwrite derived model campaign_id fields during migration
    # Will rename to "campaign_id" (and drop "campaign" field) after migration
    tunic_campaign_id = models.IntegerField(blank=True, null=True, default=None)
    # Property-specific custom configuration
    config = JSONField(default={}, blank=True)

    objects = SpecialCoverageManager()

    def __unicode__(self):
        return self.name

    def clean(self):
        super(SpecialCoverage, self).clean()
        self.clean_publish_dates()
        self.clean_query()
        self.clean_videos()
        self.clean_super_features()

    def clean_publish_dates(self):
        """
        If an end_date value is provided, the start_date must be less.
        """
        if self.end_date:
            if not self.start_date:
                raise ValidationError("""The End Date requires a Start Date value.""")
            elif self.end_date <= self.start_date:
                raise ValidationError("""The End Date must not precede the Start Date.""")
        if self.start_date and not self.end_date:
            raise ValidationError("""The Start Date requires an End Date.""")

    def clean_query(self):
        """
        Removes any `None` value from an elasticsearch query.
        """
        if self.query and self.query != {}:
            for key, value in self.query.items():
                if isinstance(value, list) and None in value:
                    self.query[key] = [v for v in value if v is not None]

    def clean_videos(self):
        """
        Validates that all values in the video list are integer ids and removes all None values.
        """
        if self.videos:
            self.videos = [int(v) for v in self.videos if v is not None and is_valid_digit(v)]

    def clean_super_features(self):
        """
        Removes any null & non-integer values from the super feature list
        """
        if self.super_features:
            self.super_features = [int(sf) for sf in self.super_features
                                   if sf is not None and is_valid_digit(sf)]

    def save(self, *args, **kwargs):
        """Saving ensures that the slug, if not set, is set to the slugified name."""
        self.clean()

        if not self.slug:
            self.slug = slugify(self.name)

        super(SpecialCoverage, self).save(*args, **kwargs)

        if self.query and self.query != {}:
            # Always save and require client to filter active date range
            self._save_percolator()

    def _save_percolator(self):
        """
        Saves the query field as an elasticsearch percolator
        """
        index = Content.search_objects.mapping.index
        query_filter = self.get_content(published=False).to_dict()

        q = {}

        if "query" in query_filter:
            q = {"query": query_filter.get("query", {})}
        else:
            # We don't know how to save this
            return

        # We'll need this data, to decide which special coverage section to use
        q["sponsored"] = bool(self.tunic_campaign_id)
        # Elasticsearch v1.4 percolator "field_value_factor" does not
        # support missing fields, so always need to include

        q["start_date"] = self.start_date
        q["end_date"] = self.end_date

        # Elasticsearch v1.4 percolator range query does not support DateTime range queries
        # (PercolateContext.nowInMillisImpl is not implemented).
        if q["start_date"]:
            q['start_date_epoch'] = datetime_to_epoch_seconds(q["start_date"])
        if q["end_date"]:
            q['end_date_epoch'] = datetime_to_epoch_seconds(q["end_date"])

        # Store manually included IDs for percolator retrieval scoring (boost
        # manually included content).
        if self.query:
            q['included_ids'] = self.query.get('included_ids', [])

        es.index(
            index=index,
            doc_type=".percolator",
            body=q,
            id=self.es_id
        )

    def _delete_percolator(self):
        index = Content.search_objects.mapping.index
        es.delete(index=index, doc_type=".percolator", id=self.es_id, refresh=True, ignore=404)

    def get_content(self, published=True):
        """performs es search and gets content objects
        """
        if "query" in self.query:
            q = self.query["query"]
        else:
            q = self.query
        search = custom_search_model(Content, q, published=published, field_map={
            "feature-type": "feature_type.slug",
            "tag": "tags.slug",
            "content-type": "_type"
        })
        return search

    @property
    def is_active(self):
        now = today_as_utc_datetime()
        if self.start_date and self.end_date:
            return self.start_date <= now < self.end_date
        return False

    @property
    def es_id(self):
        return "specialcoverage.{}".format(self.id)

    @property
    def contents(self):
        """performs .get_content() and caches it in a ._content property
        """
        if not hasattr(self, "_content"):
            self._content = self.get_content()
        return self._content

    @property
    def identifier(self):
        return "specialcoverage.{}".format(self.id)

    @property
    def has_pinned_content(self):
        """determines if the there is a pinned object in the search
        """
        if "query" in self.query:
            q = self.query["query"]
        else:
            q = self.query
        if "pinned_ids" in q:
            return bool(len(q.get("pinned_ids", [])))
        return False

    @property
    def custom_template_name(self):
        """
        Returns the path for the custom special coverage template we want.
        """
        base_path = getattr(settings, "CUSTOM_SPECIAL_COVERAGE_PATH", "special_coverage/custom")
        if base_path is None:
            base_path = ""
        return "{0}/{1}_custom.html".format(
            base_path, self.slug.replace("-", "_")
        ).lstrip("/")


def remove_percolator(sender, instance, *args, **kwargs):
    instance._delete_percolator()

pre_delete.connect(remove_percolator, sender=SpecialCoverage)



from django.conf.urls import patterns, url

urlpatterns = patterns(
    "",
    url(
        r"v/(?P<videohub_ref_id>\d+)/?$",
        "bulbs.videos.views.video_redirect",
        name="video_redirect"
    )
)



from django.conf import settings


class AdTargetingMiddleware(object):

    def process_template_response(self, request, response):

        if response.context_data is None:
            return response

        targeting = {}
        if "targeting" in response.context_data:
            targeting = response.context_data["targeting"]

        if "dfp_site" not in targeting:
            targeting["dfp_site"] = settings.DFP_SITE

        if request.method == "GET" and "dfptest" in request.GET:
            targeting["dfp_test"] = request.GET["dfptest"]

        response.context_data["targeting"] = targeting

        return response



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('content', '0008_content_template_choice'),
    ]

    operations = [
        migrations.AddField(
            model_name='featuretype',
            name='instant_article',
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name='content',
            name='template_choice',
            field=models.IntegerField(default=0, choices=[(0, None), (1, b'special_coverage/landing.html')]),
        ),
    ]



from django.db import models

import jsonfield
from elasticsearch_dsl import field

from bulbs.content.models import Content
from bulbs.recirc.mixins import BaseQueryMixin
from bulbs.super_features.utils import get_superfeature_choices
from bulbs.super_features.mixins import SuperFeatureMixin


GUIDE_TO_HOMEPAGE = 'GUIDE_TO_HOMEPAGE'
GUIDE_TO_ENTRY = 'GUIDE_TO_ENTRY'
BASE_CHOICES = (
    (GUIDE_TO_HOMEPAGE, 'Guide To Homepage'),
    (GUIDE_TO_ENTRY, 'Guide To Entry'),
)

SF_CHOICES = get_superfeature_choices()


class AbstractSuperFeature(models.Model):
    notes = models.TextField(null=True, blank=True, default='')
    internal_name = models.CharField(null=True, blank=True, max_length=255)
    superfeature_type = models.CharField(choices=SF_CHOICES, max_length=255)
    default_child_type = models.CharField(choices=SF_CHOICES, max_length=255, null=True, blank=True)
    data = jsonfield.JSONField()

    class Meta:
        abstract = True

    @classmethod
    def get_data_serializer(cls, name):
        from bulbs.super_features.data_serializers import (GuideToChildSerializer,
                                                           GuideToParentSerializer)

        serializer = {
            GUIDE_TO_HOMEPAGE: GuideToParentSerializer,
            GUIDE_TO_ENTRY: GuideToChildSerializer
        }.get(name, None)
        if serializer is None:
            raise KeyError('The requested SuperFeature does not have a registered serializer')

        return serializer

    def validate_data_field(self):
        Serializer = self.get_data_serializer(self.superfeature_type)  # NOQA
        Serializer(data=self.data).is_valid(raise_exception=True)

    def save(self, *args, **kwargs):
        self.full_clean()
        return super(AbstractSuperFeature, self).save(*args, **kwargs)

    def clean(self, *args, **kwargs):
        self.validate_data_field()
        return super(AbstractSuperFeature, self).clean(*args, **kwargs)


class BaseSuperFeature(SuperFeatureMixin, Content, AbstractSuperFeature, BaseQueryMixin):
    parent = models.ForeignKey('self', blank=True, null=True)
    ordering = models.IntegerField(blank=True, null=True, default=None)

    class Meta:
        unique_together = ('parent', 'ordering')

    class Mapping(Content.Mapping):
        # NOTE: parent is set to integer so DJES doesn't recurse
        parent = field.Integer()
        data = field.Object()

        class Meta:
            # Necessary to allow for our data field to store appropriately in Elasticsearch.
            # A potential alternative could be storing as a string., we should assess the value.
            dynamic = False

    @classmethod
    def get_serializer_class(cls):
        from bulbs.super_features.serializers import BaseSuperFeatureSerializer
        return BaseSuperFeatureSerializer



from django.shortcuts import get_object_or_404

from rest_framework import views, status
from rest_framework.response import Response
from rest_framework.permissions import IsAdminUser

from bulbs.api.permissions import CanEditContent
from bulbs.super_features.utils import get_superfeature_model, get_superfeature_partial_serializer


SUPERFEATURE_MODEL = get_superfeature_model()
SUPERFEATURE_PARTIAL_SERIALIZER = get_superfeature_partial_serializer()


class RelationViewSet(views.APIView):

    permission_classes = (IsAdminUser, CanEditContent,)

    def get(self, request, pk):
        children = SUPERFEATURE_MODEL.objects.filter(parent__id=pk)
        result = SUPERFEATURE_PARTIAL_SERIALIZER(children, many=True)

        return Response(result.data, status=status.HTTP_200_OK)


class SetChildrenDatesViewSet(views.APIView):

    permission_classes = (IsAdminUser, CanEditContent,)

    def put(self, request, pk):
        parent = get_object_or_404(SUPERFEATURE_MODEL, pk=pk)

        if not parent.is_published:
            return Response(
                {'detail': 'Parent publish date is not set'},
                status=status.HTTP_400_BAD_REQUEST
            )

        children = SUPERFEATURE_MODEL.objects.filter(parent__id=pk)
        for child in children:
            child.published = parent.published
            child.save()

        return Response(status=status.HTTP_200_OK)



import re

from bs4 import BeautifulSoup
import six


def has_attr(attr):
    """Useful wrapper for filtering w/ BeautifulSoup 'find' methods

    ex:  tag.find('iframe', has_attr('src'))
    """
    def inner_has_attr(tag):
        return tag and tag.has_attr(attr)
    return inner_has_attr


def prepare_iframe(iframe):
    """Standard IFRAME cleanup when raw iframe is used as output."""

    # Strip out style attribute
    if iframe.has_attr('style'):
        del iframe.attrs['style']
    # Convert to unicode (for Python 2.7, noop for 3.x)
    return six.text_type(iframe)


def parse_betty(tag):
    if (tag.name == 'div' and
        'image' in tag.get('class', {}) and
            tag.attrs.get('data-type') == 'image' and
            tag.has_attr('data-image-id') and
            tag.has_attr('data-format')):
        caption = tag.find('span', class_='caption')
        return {'betty': {'image_id': tag.attrs['data-image-id'],
                          'caption': caption.text if caption else '',
                          'format': tag.attrs['data-format'],
                          'ratio': tag.attrs.get('data-crop', '16x9')}}


def parse_facebook(tag):
    if tag.name == 'div' and tag.attrs.get('data-type') == 'embed':
        iframe = tag.find('iframe')
        if iframe:
            # Identify by 'src' URL
            src = iframe.attrs.get('src', '')
            if re.match('https?://www.facebook.com/plugins/', src):
                return {'facebook': {'iframe': prepare_iframe(iframe)}}


def parse_instagram(tag):
    INSTAGRAM_ID_REGEX = re.compile('https?://www.instagram.com/p/([^/]+)/')
    INSTAGRAM_HTML_ID_REGEX = re.compile('instagram.com/p/([^/]+)/')

    if tag.name == 'div' and tag.attrs.get('data-type') == 'embed':
        # IFRAME
        iframe = tag.find('iframe', 'instagram-media', has_attr('src'))
        if iframe:
            m = INSTAGRAM_ID_REGEX.match(iframe['src'])
            if m:
                return {'instagram': {'instagram_id': m.group(1)}}

        # Blockquote
        blockquote = tag.find('blockquote', 'instagram-media')
        if blockquote:
            for a in blockquote.findAll('a'):
                if a.has_attr('href'):
                    m = INSTAGRAM_ID_REGEX.match(a['href'])
                    if m:
                        return {'instagram': {'instagram_id': m.group(1)}}
    # Div embed
    elif tag.name == 'div' and tag.attrs.get('data-type') == 'embed-instagram':
        div = tag.find('div', 'embed-container')
        m = INSTAGRAM_HTML_ID_REGEX.search(div['instagram-embed-html'])

        if m:
            return {'instagram': {'instagram_id': m.group(1)}}


def parse_text(tag):
    if tag.name in ['p', 'blockquote', 'ol', 'ul', 'h3', 'h4']:
        return {'text': {'raw': six.text_type(tag)}}


def parse_twitter(tag):
    if tag.name == 'div' and tag.attrs.get('data-type') == 'embed':
        for class_ in ['twitter-video', 'twitter-tweet']:
            blockquote = tag.find('blockquote', class_=class_)
            if blockquote:
                return {'twitter': {'blockquote': six.text_type(blockquote)}}


def parse_youtube(tag):
    if tag.name == 'div':
        # No-IFRAME - grab ID from attribute
        if tag.attrs.get('data-type') == 'youtube':
            video_id = tag.attrs.get('data-youtube-id')
            if video_id:
                return {'youtube': {'video_id': video_id}}
        # IFRAME - parse ID from 'src' attribute
        if tag.attrs.get('data-type') == 'embed':
            iframe = tag.find('iframe')
            if iframe and iframe.has_attr('src'):
                m = re.match('https?://www.youtube.com/embed/(.+)', iframe.attrs['src'])
                if m:
                    return {'youtube': {'video_id': m.group(1)}}


def parse_onion_video(tag):
    if tag.name == 'div' and tag.attrs.get('data-type') == 'embed':
        iframe = tag.find('iframe', class_='onionstudios-playlist')
        if iframe:
            return {'onion_video': {'iframe': prepare_iframe(iframe)}}


def parse_vimeo(tag):
    if tag.name == 'div' and tag.attrs.get('data-type') == 'embed':
        iframe = tag.find('iframe')
        if iframe:
            # Identify by 'src' URL
            src = iframe.attrs.get('src', '')
            if re.match('https?://player.vimeo.com/video/', src):
                return {'vimeo': {'iframe': prepare_iframe(iframe)}}


def parse_soundcloud(tag):
    if tag.name == 'div' and tag.attrs.get('data-type') == 'embed':
        iframe = tag.find('iframe')
        if iframe:
            # Identify by 'src' URL
            src = iframe.attrs.get('src', '')
            if re.match('https?://w.soundcloud.com/player/', src):
                return {'soundcloud': {'iframe': prepare_iframe(iframe)}}


def parse_imgur(tag):
    if tag.name == 'div' and tag.attrs.get('data-type') == 'embed':
        iframe = tag.find('iframe', class_='imgur-embed-iframe-pub')
        if iframe:
            return {'imgur': {'iframe': prepare_iframe(iframe)}}


PARSERS = [
    # Sorted by precedence (initially alphabically since no precedence required for now)
    parse_betty,
    parse_facebook,
    parse_imgur,
    parse_instagram,
    parse_onion_video,
    parse_soundcloud,
    parse_twitter,
    parse_vimeo,
    parse_youtube,
    # Simple <p> tags have lowest priority
    parse_text,
]


def parse_tag(tag):
    """Check a single tag against all possible parsers. First match wins."""
    for parser in PARSERS:
        match = parser(tag)
        if match:
            return match


def parse_children(parent):
    """Recursively parse child tags until match is found"""

    components = []
    for tag in parent.children:
        matched = parse_tag(tag)
        if matched:
            components.append(matched)
        elif hasattr(tag, 'contents'):
            components += parse_children(tag)
    return components


def parse_body(html):
    """Parse entire content blob of HTML into ordered list of components for renderer.

    Example return value:

        [{'text': {'raw': '<p>One paragraph</p>'},
         {'youtube': {'video_id': '12345'}},
         {'youtube': {'video_id': 'abcdefg'}}]

    """
    return parse_children(BeautifulSoup(html))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('special_coverage', '0002_auto_20150318_2110'),
    ]

    operations = [
        migrations.AlterField(
            model_name='specialcoverage',
            name='campaign',
            field=models.ForeignKey(on_delete=django.db.models.deletion.SET_NULL, default=None, blank=True, to='campaigns.Campaign', null=True),
            preserve_default=True,
        ),
    ]



import re

from rest_framework import serializers

from djbetty.serializers import ImageFieldSerializer

from bulbs.content.serializers import ContentSerializer
from .models import Poll, Answer


class AnswerSerializer(serializers.ModelSerializer):
    answer_image = ImageFieldSerializer(required=False, allow_null=True,)
    poll = serializers.PrimaryKeyRelatedField(
        queryset=Poll.objects.all(),
        required=False
    )

    class Meta:
        model = Answer
        fields = ('id', 'answer_text', 'poll', 'answer_image',)


class PollSerializer(ContentSerializer):
    poll_image = ImageFieldSerializer(required=False)
    answers = AnswerSerializer(many=True, read_only=True)

    class Meta:
        model = Poll
        exclude = ('thumbnail_override',)


class PollPublicSerializer(PollSerializer):
    def to_representation(self, poll):
        representation = super(PollSerializer, self).to_representation(poll)
        sodahead_data = poll.get_sodahead_data()
        representation['total_votes'] = sodahead_data['poll']['totalVotes']
        index = 0
        for answer in poll.answers.all():
            sodahead_index = int(re.findall(r'\d+', answer.sodahead_answer_id)[0]) - 1
            sodahead_answer = sodahead_data['poll']['answers'][sodahead_index]
            representation['answers'][index]['sodahead_id'] = sodahead_answer['id']
            representation['answers'][index]['total_votes'] = sodahead_answer['totalVotes']
            if answer.answer_image:
                representation['answers'][index]['answer_image_url'] = answer.answer_image.get_crop_url(ratio='1x1')
            index += 1

        return representation



from rest_framework import serializers

from djbetty.serializers import ImageFieldSerializer

from bulbs.utils.data_serializers import EntrySerializer


class GuideToChildSerializer(serializers.Serializer):
    entries = EntrySerializer(many=True, required=False, child_label="entry")


class GuideToParentSerializer(serializers.Serializer):
    sponsor_brand_messaging = serializers.CharField(required=False)
    sponsor_product_shot = ImageFieldSerializer(
        required=False,
        label="Sponsor Product Shot (1x1 Image)"
    )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('notifications', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notification',
            name='headline',
            field=models.CharField(blank=True, null=True, max_length=512),
        ),
    ]



"""Module to generate and send a report to contributors containing a log of their contributions."""
import logging

from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.mail import EmailMultiAlternatives
from django.template import loader
from django.utils import timezone

logger = logging.getLogger(__name__)

User = get_user_model()


# Define constants.
CONTRIBUTION_SETTINGS = getattr(settings, "CONTRIBUTIONS", {})
EMAIL_SETTINGS = CONTRIBUTION_SETTINGS.get("EMAIL", {})
DEFAULT_SUBJECT = "Contribution Report."
TEMPLATE = "reporting/__contribution_report.html"


class ContributorReport(object):
    """Generate a contributor email report."""

    def __init__(self, contributor, **kwargs):
        self.contributor = contributor

        self.now = timezone.now()
        self.month = kwargs.get("month", self.now.month)
        self.year = kwargs.get("year", self.now.year)

        self._contributions = None
        self._line_items = None
        self._deadline = kwargs.get("deadline")
        self._start = kwargs.get("start")
        self._end = kwargs.get("end")

        self._total = 0

    def send(self):
        if self.is_valid():
            mail = EmailMultiAlternatives(
                subject=EMAIL_SETTINGS.get("SUBJECT", DEFAULT_SUBJECT),
                from_email=EMAIL_SETTINGS.get("FROM"),
                bcc=EMAIL_SETTINGS.get("BCC"),
                headers={"Reply-To": EMAIL_SETTINGS.get("REPLY_TO")}
            )
            mail.attach_alternative(self.get_body(), "text/html")
            if EMAIL_SETTINGS.get("ACTIVE", False):
                mail.to = [self.contributor.email]
            else:
                mail.to = EMAIL_SETTINGS.get("TO")
            mail.send()
        else:
            logger.error("""No email sent for {}""".format(self.contributor.get_full_name()))

    def get_body(self):
        contribution_types = {}
        for contribution in self.contributions:
            contribution_types[contribution] = contribution.content._meta.concrete_model.__name__
        context = {
            "content_type": contribution.content._meta.concrete_model.__name__,
            "contributor": self.contributor,
            "contributions": contribution_types,
            "deadline": self.deadline,
            "line_items": self.line_items,
            "total": self.total
        }
        return loader.render_to_string(TEMPLATE, context)

    def is_valid(self):
        """returns `True` if the report should be sent."""
        if not self.total:
            return False
        if not self.contributor.freelanceprofile.is_freelance:
            return False
        return True

    @property
    def contributions(self):
        """Apply a datetime filter against the contributor's contribution queryset."""
        if self._contributions is None:
            self._contributions = self.contributor.contributions.filter(
                content__published__gte=self.start,
                content__published__lt=self.end
            )
        return self._contributions

    @property
    def line_items(self):
        """Apply a datetime filter against the contributors's line item queryset."""
        if self._line_items is None:
            self._line_items = self.contributor.line_items.filter(
                payment_date__range=(self.start, self.end)
            )
        return self._line_items

    @property
    def total(self):
        if self._total == 0 and self.contributions:
            self._total += sum(
                [contribution.pay for contribution in self.contributions if contribution.pay]
            )
            self._total += sum(
                [line_item.amount for line_item in self.line_items if line_item.amount]
            )
        return self._total

    @property
    def deadline(self):
        """Return next day as deadline if no deadline provided."""
        if not self._deadline:
            self._deadline = self.now + timezone.timedelta(days=1)
        return self._deadline

    @property
    def start(self):
        if not self._start:
            self._start = timezone.datetime(day=1, month=self.month, year=self.year)
        return self._start

    @property
    def end(self):
        if not self._end:
            next_month = (self.start.month + 1) % 12
            year = self.start.year
            if next_month == 1:
                year += 1
            elif next_month == 0:
                next_month = 12
            self._end = timezone.datetime(day=1, month=next_month, year=year)
        return self._end


class EmailReport(object):
    """Generate an email report for contributors."""

    def __init__(self, **kwargs):
        # if "start" in kwargs:
        self.now = timezone.now()
        self.month = kwargs.get("month", self.now.month)
        self.year = kwargs.get("year", self.now.year)

        self._contributors = []
        self._deadline = kwargs.get("deadline")
        self._start = kwargs.get("start")
        self._end = kwargs.get("end")

    def send_contributor_email(self, contributor):
        """Send an EmailMessage object for a given contributor."""
        ContributorReport(
            contributor,
            month=self.month,
            year=self.year,
            deadline=self._deadline,
            start=self._start,
            end=self._end
        ).send()

    def send_mass_contributor_emails(self):
        """Send report email to all relevant contributors."""
        # If the report configuration is not active we only send to the debugging user.
        for contributor in self.contributors:
            if contributor.email not in EMAIL_SETTINGS.get("EXCLUDED", []):
                self.send_contributor_email(contributor)

    def get_contributors(self):
        """Return a list of contributors with contributions between the start/end dates."""
        return User.objects.filter(
            freelanceprofile__is_freelance=True
        ).filter(
            contributions__content__published__gte=self.start,
            contributions__content__published__lt=self.end
        ).distinct()

    @property
    def contributors(self):
        """Property to retrieve or access the list of contributors."""
        if not self._contributors:
            self._contributors = self.get_contributors()
        return self._contributors

    @property
    def start(self):
        if not self._start:
            self._start = timezone.datetime(day=1, month=self.month, year=self.year)
        return self._start

    @property
    def end(self):
        if not self._end:
            next_month = (self.start.month + 1) % 12
            year = self.start.year
            if next_month == 1:
                year += 1
            elif next_month == 0:
                next_month = 12
            self._end = timezone.datetime(day=1, month=next_month, year=year)
        return self._end


def send_byline_email(content, removed_bylines):
    context = {
        "content": content,
        "bylines": removed_bylines,
        "contributions": content.contributions.all()
    }
    body = loader.render_to_string("reporting/__byline_report.html", context)
    mail = EmailMultiAlternatives(
        subject="Byline modified for article, {}".format(content.title.encode("ascii", "ignore")),
        from_email=EMAIL_SETTINGS.get("FROM"),
    )
    mail.attach_alternative(body, "text/html")
    mail.to = EMAIL_SETTINGS.get("BYLINE_RECIPIENTS", [])
    mail.send()



import django_filters
from rest_framework import filters

from bulbs.super_features.utils import get_superfeature_model

SUPERFEATURE_MODEL = get_superfeature_model()


def filter_status(queryset, value):
    if not value:
        return queryset
    else:
        # NOTE: this list comprehension is happening because
        # status is a property, not a model field
        # see: http://stackoverflow.com/a/1205416
        return [sf for sf in queryset
                if sf.status.lower() == value.lower()]


class SuperFeatureFilter(filters.FilterSet):
    status = django_filters.CharFilter(action=filter_status)

    class Meta:
        model = SUPERFEATURE_MODEL
        fields = ['status']



"""Common utilities for special coverage behaviors."""
from django.conf import settings
from django.utils import timezone

from bulbs.content.filters import Published
from .models import SpecialCoverage
from .search import SearchParty


def get_sponsored_special_coverages():
    """:returns: Django query for all active special coverages with active campaigns."""
    now = timezone.now()
    return SpecialCoverage.objects.filter(
        tunic_campaign_id__isnull=False,
        start_date__lte=now,
        end_date__gt=now
    )


def get_sponsored_special_coverage_query(only_recent=False):
    """
    Reference to all SpecialCovearge queries.

    :param only_recent: references RECENT_SPONSORED_OFFSET_HOURS from django settings.
        Used to return sponsored content within a given configuration of hours.
    :returns: Djes.LazySearch query matching all active speical coverages.
    """
    special_coverages = get_sponsored_special_coverages()
    es_query = SearchParty(special_coverages).search()
    if only_recent:
        offset = getattr(settings, "RECENT_SPONSORED_OFFSET_HOURS", 0)
        es_query = es_query.filter(
            Published(after=timezone.now() - timezone.timedelta(hours=offset))
        )
    return es_query



from django.apps import apps
from django.core.management.base import BaseCommand

from djes.models import Indexable


class Command(BaseCommand):

    def handle(self, *args, **kwargs):
        app = apps.get_app('contributions')
        if not app:
            return

        for model in apps.get_models(app):
            if issubclass(model, Indexable):
                for instance in model.objects.all():
                    instance.index()



from copy import deepcopy

from django.conf import settings
from django.db import models

from elasticsearch import Elasticsearch
from elasticsearch_dsl.filter import Ids
from json_field import JSONField

from bulbs.content.custom_search import custom_search_model
from bulbs.content.filters import FeatureTypeBoost, TagBoost
from bulbs.content.models import Content


es = Elasticsearch(settings.ES_CONNECTIONS["default"]["hosts"])


class BaseQueryMixin(models.Model):

    query = JSONField(default={}, blank=True)

    class Meta:
        abstract = True

    def clean(self):
        super(BaseQueryMixin, self).clean()
        self.clean_query()

    def clean_query(self):
        """
        Removes any `None` value from an elasticsearch query.
        """
        if self.query:
            for key, value in self.query.items():
                if isinstance(value, list) and None in value:
                    self.query[key] = [v for v in value if v is not None]

    def save(self, *args, **kwargs):
        self.clean()
        super(BaseQueryMixin, self).save()

    def get_recirc_content(self, published=True, count=3):
        """gets the first 3 content objects in the `included_ids`
        """
        query = self.get_query()

        # check if query has included_ids & if there are any ids in it,
        # in case the ids have been removed from the array
        if not query.get('included_ids'):
            qs = Content.search_objects.search()
            qs = qs.query(
                TagBoost(slugs=self.tags.values_list("slug", flat=True))
            ).filter(
                ~Ids(values=[self.id])
            ).sort(
                "_score"
            )
            return qs[:count]

        # NOTE: set included_ids to just be the first 3 ids,
        # otherwise search will return last 3 items
        query['included_ids'] = query['included_ids'][:count]
        search = custom_search_model(Content, query, published=published, field_map={
            "feature_type": "feature_type.slug",
            "tag": "tags.slug",
            "content-type": "_type"
        })
        return search

    def get_full_recirc_content(self, published=True):
        """performs es search and gets all content objects
        """
        q = self.get_query()
        search = custom_search_model(Content, q, published=published, field_map={
            "feature_type": "feature_type.slug",
            "tag": "tags.slug",
            "content-type": "_type"
        })
        return search

    def get_inline_recirc_content(self, published=True, count=3):
        qs = Content.search_objects.search()
        qs = qs.query(
            TagBoost(slugs=self.tags.values_list("slug", flat=True))
        ).query(
            FeatureTypeBoost(slugs=["video"])
        ).filter(
            ~Ids(values=[self.id])
        ).sort(
            "_score"
        )

        return qs[:count]

    def get_query(self):
        if "query" in self.query:
            q = self.query["query"]
        else:
            q = self.query
        return deepcopy(q)

    @property
    def contents(self):
        """performs .get_content() and caches it in a ._content property
        """
        if not hasattr(self, "_content"):
            self._content = self.get_content()
        return self._content

    @property
    def has_pinned_content(self):
        """determines if the there is a pinned object in the search
        """
        q = self.get_query()
        if "pinned_ids" in q:
            return bool(len(q.get("pinned_ids", [])))
        return False



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import djbetty.fields
import json_field.fields


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Section',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(unique=True, max_length=255)),
                ('slug', models.SlugField(unique=True, max_length=255, blank=True)),
                ('description', models.TextField(default=b'', blank=True)),
                ('embed_code', models.TextField(default=b'', blank=True)),
                ('section_logo', djbetty.fields.ImageField(default=None, null=True, blank=True)),
                ('twitter_handle', models.CharField(max_length=255, blank=True)),
                ('promoted', models.BooleanField(default=False)),
                ('query', json_field.fields.JSONField(default={}, help_text='Enter a valid JSON object', blank=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
    ]



from django.conf import settings
from django.db import models

from djbetty import ImageField
from djes.models import Indexable

from bulbs.content.models import ElasticsearchImageField


class Notification(Indexable):
    """Model for site notifications."""

    # Internal
    internal_title = models.CharField(max_length=512)
    created_on = models.DateTimeField(auto_now_add=True)

    # Editorial
    headline = models.CharField(max_length=512, null=True, blank=True)
    is_published = models.BooleanField(default=False)
    body = models.TextField(null=True, blank=True)
    image = ImageField(blank=True, null=True)
    clickthrough_url = models.URLField(blank=True, null=True)
    clickthrough_cta = models.CharField(blank=True, null=True, max_length=30)

    class Mapping:
        image = ElasticsearchImageField()

    def get_clickthrough_cta(self):
        """Return fallback cta, configured by the given property cms."""
        if not self.clickthrough_cta:
            return getattr(settings, "NOTIFICATION_CLICKTHROUGH_CTA", "Read More...")
        return self.clickthrough_cta
